# -*- coding: utf-8 -*-

# Simple dummy Sphinx configuration

source_suffix = '.rst'
master_doc = 'index'

# General information about the project.
project = u'rst-sphinx'
copyright = u'2014, Foo'
version = '0'
release = '0'



# -*- coding: utf-8; -*-

import sys

from . import antigravit  # Typo intended!

class Spam(object):

    def withEggs(self, number_of_eggs):
        return map(str, range(number_of_eggs))

    def with_ham(self, blubb = None):
        if blubb:
            if sys.python_version[0] > 3:
                print('yes')
            else:
                print('no')
        else:
            return 'foo'


antigravity()



# Copyright (C) 2016 Sebastian Wiesner and Flycheck contributors

# This file is not part of GNU Emacs.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from string import Template

import requests
from docutils import nodes
from sphinx.roles import XRefRole
from sphinx.util import ws_re, logging

logger = logging.getLogger(__name__)

# Regular expression object to parse the contents of an Info reference
# role.
INFO_RE = re.compile(r'\A\((?P<manual>[^)]+)\)(?P<node>.+)\Z')


class InfoNodeXRefRole(XRefRole):
    """A role to reference a node in an Info manual."""

    innernodeclass = nodes.emphasis

    def process_link(self, env, refnode, has_explicit_title, title, target):
        """Process the link created by this role.

        Swap node and manual name, to more closely match the look of references
        in Texinfo.

        """
        # Normalize whitespace in info node targets
        target = ws_re.sub(' ', target)
        refnode['has_explicit_title'] = has_explicit_title
        if not has_explicit_title:
            match = INFO_RE.match(target)
            if match:
                # Swap title and node to create a title like info does
                title = '{0}({1})'.format(match.group('node'),
                                          match.group('manual'))
        return title, target


def node_encode(char):
    if char.isalnum():
        return char
    elif char == ' ':
        return '-'
    else:
        return '_00{:02x}'.format(ord(char))


def expand_node_name(node):
    """Expand ``node`` for use in HTML.

    ``node`` is the name of a node as string.

    Return a pair ``(filename, anchor)``, where ``filename`` is the base-name
    of the corresponding file, sans extension, and ``anchor`` the HTML anchor.

    See
    http://www.gnu.org/software/texinfo/manual/texinfo/html_node/HTML-Xref-Node-Name-Expansion.html.

    """
    if node == 'Top':
        return ('index', 'Top')
    else:
        normalized = ws_re.sub(' ', node.strip())
        encoded = ''.join(node_encode(c) for c in normalized)
        prefix = 'g_t' if not node[0].isalpha() else ''
        return (encoded, prefix + encoded)


class HTMLXRefDB(object):
    """Cross-reference database for Info manuals."""

    #: URL of the htmlxref database of GNU Texinfo
    XREF_URL = 'http://ftpmirror.gnu.org/texinfo/htmlxref.cnf'

    #: Regular expression to parse entries from an xref DB
    XREF_RE = re.compile(r"""
^\s*
(?:
(?P<comment>[#].*) |
(?P<substname>\w+)\s*=\s*(?P<substurl>\S+) |
(?P<manname>\w+)\s*(?P<mantype>node|mono)\s*(?P<manurl>\S+)
)
\s*$""", re.VERBOSE)

    @classmethod
    def parse(cls, htmlxref):
        substitutions = {}
        manuals = {}
        for line in htmlxref.splitlines():
            match = cls.XREF_RE.match(line)
            if match:
                if match.group('substname'):
                    url = Template(match.group('substurl')).substitute(
                        substitutions)
                    substitutions[match.group('substname')] = url
                elif (match.group('manname')
                      and match.group('mantype') == 'node'):
                    url = Template(match.group('manurl')).substitute(
                        substitutions)
                    manuals[match.group('manname')] = url
        return cls(manuals)

    def __init__(self, entries):
        """Initialize the HTMLXrefDB object with the provided entries."""
        self.entries = entries

    def resolve(self, manual, node):
        manual_url = self.entries.get(manual)
        if not manual_url:
            return None
        else:
            filename, anchor = expand_node_name(node)
            return manual_url + filename + '.html#' + anchor


def update_htmlxref(app):
    if not isinstance(getattr(app.env, 'info_htmlxref', None), HTMLXRefDB):
        logger.info('fetching Texinfo htmlxref database from {0}... '.format(
            HTMLXRefDB.XREF_URL))
        try:
            app.env.info_htmlxref = HTMLXRefDB.parse(
                requests.get(HTMLXRefDB.XREF_URL).text)
        except requests.exceptions.ConnectionError:
            logger.warning('Failed to load xref DB.  '
                           'Info references will not be resolved')
            app.env.info_htmlxref = None


def resolve_info_references(app, _env, refnode, contnode):
    """Resolve Info references.

    Process all :class:`~sphinx.addnodes.pending_xref` nodes whose ``reftype``
    is ``infonode``.

    Replace the pending reference with a :class:`~docutils.nodes.reference`
    node, which references the corresponding web URL, as stored in the database
    referred to by :data:`HTMLXREF_URL`.

    """
    if refnode['reftype'] != 'infonode':
        return None

    target = ws_re.sub(' ', refnode['reftarget'])
    match = INFO_RE.match(target)
    if not match:
        logger.warning('Invalid info target: {0}'.format(target),
                       location=(refnode.source, refnode.line))
        return contnode

    manual = match.group('manual')
    node = match.group('node')

    xrefdb = app.env.info_htmlxref
    if xrefdb:
        uri = xrefdb.resolve(manual, node)
        if not uri:
            message = 'Cannot resolve info manual {0}'.format(manual)
            logger.warning(message, location=(refnode.source, refnode.line))
            return contnode
        else:
            reference = nodes.reference('', '', internal=False,
                                        refuri=uri, reftitle=target)
            reference += contnode
            return reference
    else:
        # Without an xref DB we're unable to resolve any info references
        return None


def setup(app):
    app.add_role('infonode', InfoNodeXRefRole())
    app.connect(str('builder-inited'), update_htmlxref)
    app.connect(str('missing-reference'), resolve_info_references)
    return {'version': '0.1', 'parallel_read_safe': True}



#!/usr/bin/env python3
# Copyright (C) 2017 Flycheck contributors
# Copyright (C) 2016 Sebastian Wiesner and Flycheck contributors

# This file is not part of GNU Emacs.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import subprocess
from datetime import date
from collections import namedtuple
from pathlib import Path

import requests
from git import Repo


SOURCE_DIR = Path(__file__).resolve().parent.parent
FLYCHECK_EL = SOURCE_DIR.joinpath('flycheck.el')
CHANGELOG = SOURCE_DIR.joinpath('CHANGES.rst')

TRAVIS_ENDPOINT = 'https://api.travis-ci.org/repos/flycheck/flycheck'

VERSION_HEADER_RE = re.compile(
    r'^(?P<label>;;\s*Version:\s*)(?P<value>\S+)\s*$',
    re.MULTILINE)


class CannotReleaseError(Exception):
    pass


class Version(namedtuple('Version', 'version is_snapshot')):

    RE = re.compile(r'^(?P<version>\d+)(?:(?P<snapshot>-cvs))?$')

    @classmethod
    def fromstring(cls, s):
        match = cls.RE.match(s)
        if not match:
            raise ValueError('Not a version: {}'.format(s))
        return cls(version=int(match.group('version')),
                   is_snapshot=match.group('snapshot') is not None)

    def __str__(self):
        if self.is_snapshot:
            return '{}-cvs'.format(self.version)
        else:
            return str(self.version)

    @property
    def is_released(self):
        return not self.is_snapshot

    def bump(self):
        if self.is_snapshot:
            # If snapshot, then bump to release version by dropping the
            # snapshot indicator
            return self._replace(is_snapshot=False)
        else:
            # If release bump to the next snapshot version
            return self._replace(version=self.version + 1, is_snapshot=True)


class BuildState(namedtuple('BuildState', 'commit state')):

    @classmethod
    def get_from_travis_ci(cls):
        response = requests.get(
            TRAVIS_ENDPOINT + '/branches/master',
            headers={'Accept': 'application/vnd.travis-ci.2+json'}).json()
        return cls(commit=response['commit']['sha'],
                   state=response['branch']['state'])


def read_version_from_library_header(path):
    contents = path.read_text()
    match = VERSION_HEADER_RE.search(contents)
    if match:
        return Version.fromstring(match.group('value'))
    else:
        raise ValueError('Could not find version header in {}'.format(path))


def set_version_in_library_header(path, version):
    contents = path.read_text()
    path.write_text(VERSION_HEADER_RE.sub(
        r'\g<label>{}'.format(version), contents))


def finalise_relase_in_changelog(path, version, date):
    lines = path.read_text().splitlines()
    if not lines[0].endswith(' (in development)'):
        raise ValueError('Failed to find snapshot header in {}'.format(path))
    new_header = '{} ({})'.format(version, date.strftime('%b %d, %Y'))
    header_underline = '=' * len(new_header)
    path.write_text(
        '\n'.join([new_header, header_underline] + lines[2:]) + '\n')


def add_snapshot_to_changelog(path, version):
    header = '{} (in development)'.format(version)
    contents = path.read_text()
    underline = '=' * len(header)
    path.write_text('{}\n{}\n\n{}'.format(header, underline, contents))


def commit_and_push_release(repo, version):
    repo.index.add(str(p) for p in [FLYCHECK_EL, CHANGELOG])
    repo.index.commit('Release version {}'.format(version))
    repo.create_tag(str(version), message='Flycheck {}'.format(version),
                    sign=True)
    repo.remotes.origin.push('master', follow_tags=True)


def commit_and_push_snapshot(repo):
    repo.index.add(str(p) for p in [FLYCHECK_EL, CHANGELOG])
    repo.index.commit('Bump version in master')
    repo.remotes.origin.push('master')


def build_dist():
    subprocess.run(['cask', 'package'], cwd=str(SOURCE_DIR), check=True)


def ask_yes_or_no(prompt):
    return input(prompt).lower() == 'y'


def ensure_can_make_release(repo):
    if repo.head.ref != repo.refs.master:
        raise CannotReleaseError(
            'Cannot make release from branch {}.'
            ' Switch to master!'.format(repo.head.ref))
    if repo.is_dirty(untracked_files=True):
        raise CannotReleaseError(
            'Cannot release from dirty working directory.'
            ' Please commit or stash all changes!')
    state = BuildState.get_from_travis_ci()
    if state.commit != repo.head.ref.object.hexsha:
        raise CannotReleaseError(
            'HEAD not tested on Travis CI.\n'
            'Please push your changes and wait for the build to complete.')
    if state.state != 'passed':
        raise CannotReleaseError(
            'Build not passed (state: {})\n'
            'Wait for the build to finish or fix the error!'.format(
                state.state))


def main():
    try:
        repo = Repo(str(SOURCE_DIR))
        ensure_can_make_release(repo)

        current_version = read_version_from_library_header(FLYCHECK_EL)
        next_version = current_version.bump()

        if not ask_yes_or_no('Releasing Flycheck {}, '
                             'are you sure? [yn] '.format(next_version)):
            raise CannotReleaseError('Aborted')

        set_version_in_library_header(FLYCHECK_EL, next_version)
        finalise_relase_in_changelog(CHANGELOG, next_version, date.today())
        commit_and_push_release(repo, next_version)
        build_dist()

        # Now bump to next snapshot version
        next_snapshot = next_version.bump()
        set_version_in_library_header(FLYCHECK_EL, next_snapshot)
        add_snapshot_to_changelog(CHANGELOG, next_snapshot)
        commit_and_push_snapshot(repo)

        print('Flycheck {} out now, new snapshot {}! Please'.format(
            next_version, next_snapshot))
        print("""
* add information about the release to https://github.com/flycheck/flycheck/releases/edit/{0}
* upload `dist/flycheck-{0}.tar,
* enable version {0} on https://readthedocs.org/dashboard/flycheck/versions/, and
* announce the release in the flycheck/flycheck Gitter channel.
""".format(next_version))       # noqa: E501

    except CannotReleaseError as error:
        sys.exit(str(error))


if __name__ == '__main__':
    main()



# Copyright (C) 2017, 2018 Flycheck contributors
# Copyright (C) 2016 Sebastian Wiesner and Flycheck contributors

# This file is not part of GNU Emacs.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import os
from pathlib import Path
from docutils import nodes
from docutils.statemachine import ViewList
from docutils.transforms import Transform
from docutils.parsers.rst import Directive, directives
from sphinx import addnodes
from sphinx.util.nodes import set_source_info, process_index_entry
from sphinx.util import logging

logger = logging.getLogger(__name__)

sys.path.append(str(Path(__file__).parent))

ON_RTD = os.environ.get('READTHEDOCS', None) == 'True'

needs_sphinx = '1.3'
extensions = [
    'sphinx.ext.intersphinx',
    'sphinx.ext.extlinks',
    'sphinx.ext.todo',
    # Domain for Emacs Lisp
    'elisp',
    # Cross-references to info nodes
    'info'
]

# Project metadata
project = 'Flycheck'
copyright = ' 2014-2017, Sebastian Wiesner and Flycheck contributors'
author = 'Sebastian Wiesner'


def read_version():
    """Extract version number from ``flycheck.el`` and return it as string."""
    version_pattern = re.compile(r'^;;\s*Version:\s+(\d.+)$', re.MULTILINE)
    flycheck = Path(__file__).resolve().parent.parent.joinpath('flycheck.el')
    with flycheck.open(encoding='utf-8') as source:
        match = version_pattern.search(source.read())
        if match:
            return match.group(1)
        else:
            raise ValueError('Failed to parse Flycheck version from '
                             'Version: of flycheck.el')


def read_minimum_emacs_version():
    """Extract minimum Emacs version from ``flycheck.el``."""
    version_pattern = re.compile(
        r'^;; Package-Requires:.*\(emacs\s*"([^"]+)"\).*$', re.MULTILINE)
    flycheck = Path(__file__).resolve().parent.parent.joinpath('flycheck.el')
    with flycheck.open(encoding='utf-8') as source:
        match = version_pattern.search(source.read())
        if match:
            return match.group(1)
        else:
            raise ValueError('Vailed to parse minimum Emacs version from '
                             'Package-Requires of flycheck.el!')


release = read_version()
version = '.'.join(release.split('.')[:2])

# Source settings
source_suffix = '.rst'
master_doc = 'index'

rst_prolog = """\
.. role:: elisp(code)
   :language: elisp

.. |min-emacs| replace:: {emacs_version}+
""".format(emacs_version=read_minimum_emacs_version())

# Build settings
exclude_patterns = ['_build']
default_role = 'any'
primary_domain = 'el'
templates_path = ['_templates']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Warn about all undefined references, but exclude references to built-in
# symbols which we don't document here.
# TODO: Resolve built-in symbols to the Emacs Lisp references?
nitpicky = True
nitpick_ignore = [
    ('any', 'default-directory'),
    ('any', 'package-initialize'),
    ('any', 'package-archives'),
    ('any', 'user-init-file'),
    ('any', 'user-emacs-directory'),
    ('any', 'check-declare-file'),
    ('any', 'declare-function'),
    ('any', 'exec-path'),
    ('any', 'sh-shell'),
    ('any', 'rx'),
]

# HTML settings
html_theme = 'alabaster'
html_theme_options = {
    'logo': 'logo.png',
    'logo_name': False,
    'description': 'Syntax checking for GNU Emacs',
    'github_user': 'flycheck',
    'github_repo': 'flycheck',
    'github_type': 'star',
    'github_banner': True,
    'travis_button': False,
}
html_sidebars = {
    '**': [
        'about.html',
        'tables.html',
        'navigation.html',
        'relations.html',
        'searchbox.html',
    ]
}
html_static_path = ['_static']
html_favicon = '_static/favicon.ico'

# Ignore localhost when checking links
linkcheck_ignore = [r'http://localhost:\d+/?']

# Cross-reference remote Sphinx sites
intersphinx_mapping = {
    'python': ('https://docs.python.org/3.5', None)
}

extlinks = {
    'gh': ('https://github.com/%s', None),
    'flyc': ('https://github.com/flycheck/%s', None)
}

# While still have work to do :)
# FIXME: Remove when the old Texinfo manual is completed ported
todo_include_todos = True


class SupportedLanguage(Directive):

    required_arguments = 1
    final_argument_whitespace = True
    has_content = True
    option_spec = {
        'index_as': directives.unchanged
    }

    def run(self):
        language = self.arguments[0]

        indexed_languages = self.options.get('index_as') or language
        index_specs = ['pair: {}; language'.format(lang)
                       for lang in indexed_languages.splitlines()]

        name = nodes.fully_normalize_name(language)
        target = 'language-{}'.format(name)
        targetnode = nodes.target('', '', ids=[target])
        self.state.document.note_explicit_target(targetnode)

        indexnode = addnodes.index()
        indexnode['entries'] = []
        indexnode['inline'] = False
        set_source_info(self, indexnode)
        for spec in index_specs:
            indexnode['entries'].extend(process_index_entry(spec, target))

        sectionnode = nodes.section()
        sectionnode['names'].append(name)

        title, messages = self.state.inline_text(language, self.lineno)
        titlenode = nodes.title(language, '', *title)

        sectionnode += titlenode
        sectionnode += messages
        self.state.document.note_implicit_target(sectionnode, sectionnode)

        self.state.nested_parse(self.content, self.content_offset, sectionnode)

        return [indexnode, targetnode, sectionnode]


class SyntaxCheckerConfigurationFile(Directive):

    required_arguments = 1
    final_argument_whitespace = True

    def run(self):
        option = self.arguments[0]

        wrapper = nodes.paragraph()
        docname = self.state.document.settings.env.docname
        template = ViewList("""\
.. index:: single: Configuration file; {0}

.. el:defcustom:: {0}

   Configuration file for this syntax checker.  See
   :ref:`flycheck-checker-config-files`.
""".format(option).splitlines(), docname)
        self.state.nested_parse(template, self.content_offset, wrapper)

        return wrapper.children.copy()


class IssueReferences(Transform):

    ISSUE_PATTERN = re.compile(r'\[GH-(\d+)\]')
    ISSUE_URL_TEMPLATE = 'https://github.com/flycheck/flycheck/issues/{}'

    default_priority = 999

    def apply(self):
        docname = self.document.settings.env.docname
        if docname != 'changes':
            # Only transform issue references in changelo
            return

        for node in self.document.traverse(nodes.Text):
            parent = node.parent
            new_nodes = []
            last_issue_ref_end = 0
            text = str(node)
            for match in self.ISSUE_PATTERN.finditer(text):
                # Extract the text between the last issue reference and the
                # current issue reference and put it into a new text node
                head = text[last_issue_ref_end:match.start()]
                if head:
                    new_nodes.append(nodes.Text(head))
                # Adjust the position of the last issue reference in the
                # text
                last_issue_ref_end = match.end()
                # Extract the issue text and the issue number
                issuetext = match.group(0)
                issue_id = match.group(1)
                # Turn the issue into a proper reference
                refnode = nodes.reference()
                refnode['refuri'] = self.ISSUE_URL_TEMPLATE.format(issue_id)
                refnode.append(nodes.inline(
                    issuetext, issuetext, classes=['xref', 'issue']))
                new_nodes.append(refnode)

            # No issue references were found, move on to the next node
            if not new_nodes:
                continue
            # Extract the remaining text after the last issue reference
            tail = text[last_issue_ref_end:]
            if tail:
                new_nodes.append(nodes.Text(tail))
            parent.replace(node, new_nodes)


def build_offline_html(app):
    from sphinx.builders.html import StandaloneHTMLBuilder
    build_standalone = isinstance(app.builder, StandaloneHTMLBuilder)
    if app.config.flycheck_offline_html and build_standalone:
        logger.info(
            'Building offline documentation without external resources!')
        app.builder.theme_options['github_banner'] = 'false'
        app.builder.theme_options['github_button'] = 'false'


def add_offline_to_context(app, _pagename, _templatename, context, _doctree):
    # Expose offline setting in HTML context
    context['flycheck_offline_html'] = app.config.flycheck_offline_html


def setup(app):
    app.add_object_type('syntax-checker', 'checker',
                        'pair: %s; Syntax checker')
    app.add_directive('supported-language', SupportedLanguage)
    app.add_directive('syntax-checker-config-file',
                      SyntaxCheckerConfigurationFile)
    app.add_transform(IssueReferences)
    # Build offline HTML that loads no external resources, for use in 3rd party
    # packages, see https://github.com/flycheck/flycheck/issues/999
    app.add_config_value('flycheck_offline_html', False, 'html')
    app.connect('builder-inited', build_offline_html)
    app.connect('html-page-context', add_offline_to_context)
